==> wurld-0.4.tar.gz: wurld-0.4/src/gameobjs/missile.py <==
#==============================================================================
#
#  $Id$
#
"""
   A missile.
"""
#
#  Copyright (C) 2007 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#==============================================================================

import time
import wurld
from turtle import *
from util import *

# create a material to use
material = wurld.createTextureMaterial(wurld.createTexture('PlasmaVomit.png'),
				       10,
				       10
				       )

class Square:

   def __init__(self, size):
      self.size = size

   def __call__(self, turtle):
      turtle.pitch(-90)
      turtle.forward(self.size / 2, False)
      turtle.yaw(90)
      turtle.forward(self.size / 2)
      for i in range(3):
         turtle.yaw(90)
         turtle.forward(self.size)

# render the missile model

square = Square(0.5)
tool = CrossSectionTool(material, square)
turtle = Turtle(tool)

turtle.stamp()
square.size = 0
turtle.forward(1.5)
turtle.stamp()

missile = Sprite(model = tool.model, collisionAlgo = Sprite.COL_POINT)

# create a collidable and give it a high priority.  When checking for collisions
# with other objects, we want the missile's collision algorithm to have higher
# priority.
collidable = wurld.createSimpleCollidable(tool.model.getBoundingBox())
collidable.setPriority(10)
missile.addCollidable(collidable)

def createExplosionFrame(size):
   "Creates a single frame of the explosion."
   model = wurld.createModel()
   turtle = Turtle()
   turtle.pitch(-90)
   for i in xrange(6):
      for j in xrange(6):
         turtle.yaw(60)
         t = turtle.copy()
         t.forward(size)
         t.pitch(90)
         t.tool = PolyTool(material, model = model)
         t.stamp()
         for k in xrange(4):
            t.forward(size / 2.0)
            t.yaw(90)
      turtle.pitch(60)
   return model

# generate all of the explosion models
explosionAnim = [createExplosionFrame(i) for i in xrange(5)]
nullModel = wurld.createModel()

class Explosion(Sprite):

   def next(self):
      # make sure we don't run through this too fast
      t = time.time()
      if t - self.__lastTime < 0.02:
         return
      self.__lastTime = t
      
      self.__counter += 1

      # for the last frame, end the animation
      if self.__counter == 5:
         self.renderFrameCallback = None
         explosions.freeExplosion(self)
         self.model = nullModel
      else:
         # set the model to the next frame
         self.model = explosionAnim[self.__counter]
      
   def reset(self, origin):
      self.origin = origin
      self.renderFrameCallback = lambda eng, obj: self.next()
      self.model = explosionAnim[0]
      self.__counter = 0
      self.__lastTime = time.time()

class ExplosionPool:
   """
      Pool of available explosions.
   """

   def __init__(self):
      self.__free = []

   def getExplosion(self):
      """
         Returns an available explosion.  Gets a free explosion from the pool,
         or creates a new one.
      """
      if self.__free:
         return self.__free.pop(0)
      else:
         return Explosion()

   def freeExplosion(self, explosion):
      "Returns an explosion to the free pool"
      self.__free.append(explosion)

explosions = ExplosionPool()

square = Square(3)
turtle = Turtle(CrossSectionTool(material, square))
turtle.stamp()
turtle.forward(3)
turtle.stamp()
#explosionModel = turtle.tool.model

# initialize the sounds
audio = wurld.getAudioDriver()
rocketSound = audio.createSampleFromFile('sounds/rocket1i.wav')
explosionSound = audio.createSampleFromFile('sounds/r_exp3.wav')

def splat():
   global explosions
   print "SPLAT!"
   explosion = explosions.getExplosion()
   explosion.reset(missile.origin)
   player = audio.createPlayer(explosionSound)
   player.setMoveable(explosion.moveable)
   player.start()

def fire(keyCode):
   camera = wurld.getGameEngine().getCameraPtr()
   missile.origin = camera.getOrigin()
   forward = wurld.createVector(0, 0, -100)
   velocity = wurld.createVector(0, 0, 0)
   rot = camera.getRotation()
   rot.rotate(forward, velocity)
   missile.velocity = velocity
   missile.collisionCallback = splat

   # rotate the model to the inverse of the camera rotation
   modelRot = wurld.createYawRotMatrix(0)
   modelRot.makeInverse(rot)
   missile.rotation = modelRot

   # make a sound
   audio.createPlayer(rocketSound).start()

wurld.getDisplayDriver().getKeyHandler().bind(ord('a'), 
                                              wurld.createKeyCommand(fire,
                                                                     None
                                                                     )
                                              )

==> wurld-0.5.tar.gz: wurld-0.5/src/gameobjs/missile.py <==
#==============================================================================
#
#  $Id$
#
"""
   A missile.
"""
#
#  Copyright (C) 2007 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#==============================================================================

import time
import wurld
from turtle import *
from util import *

# create a material to use
material = wurld.createTextureMaterial(wurld.createTexture('PlasmaVomit.png'),
				       10,
				       10
				       )

class Square:

   def __init__(self, size):
      self.size = size

   def __call__(self, turtle):
      turtle.pitch(-90)
      turtle.forward(self.size / 2, False)
      turtle.yaw(90)
      turtle.forward(self.size / 2)
      for i in range(3):
         turtle.yaw(90)
         turtle.forward(self.size)

# render the missile model

square = Square(0.5)
tool = CrossSectionTool(material, square)
turtle = Turtle(tool)

turtle.stamp()
square.size = 0
turtle.forward(1.5)
turtle.stamp()

missile = Sprite(model = tool.model, collisionAlgo = Sprite.COL_POINT)

# create a collidable and give it a high priority.  When checking for collisions
# with other objects, we want the missile's collision algorithm to have higher
# priority.
collidable = wurld.createSimpleCollidable(tool.model.getBoundingBox())
collidable.setPriority(10)
missile.addCollidable(collidable)

def createExplosionFrame(size):
   "Creates a single frame of the explosion."
   model = wurld.createModel()
   turtle = Turtle()
   turtle.pitch(-90)
   for i in xrange(6):
      for j in xrange(6):
         turtle.yaw(60)
         t = turtle.copy()
         t.forward(size)
         t.pitch(90)
         t.tool = PolyTool(material, model = model)
         t.stamp()
         for k in xrange(4):
            t.forward(size / 2.0)
            t.yaw(90)
      turtle.pitch(60)
   return model

# generate all of the explosion models
explosionAnim = [createExplosionFrame(i) for i in xrange(5)]
nullModel = wurld.createModel()

class Explosion(Sprite):

   def next(self):
      # make sure we don't run through this too fast
      t = time.time()
      if t - self.__lastTime < 0.02:
         return
      self.__lastTime = t
      
      self.__counter += 1

      # for the last frame, end the animation
      if self.__counter == 5:
         self.renderFrameCallback = None
         explosions.freeExplosion(self)
         self.model = nullModel
      else:
         # set the model to the next frame
         self.model = explosionAnim[self.__counter]
      
   def reset(self, origin):
      self.origin = origin
      self.renderFrameCallback = lambda eng, obj: self.next()
      self.model = explosionAnim[0]
      self.__counter = 0
      self.__lastTime = time.time()

class ExplosionPool:
   """
      Pool of available explosions.
   """

   def __init__(self):
      self.__free = []

   def getExplosion(self):
      """
         Returns an available explosion.  Gets a free explosion from the pool,
         or creates a new one.
      """
      if self.__free:
         return self.__free.pop(0)
      else:
         return Explosion()

   def freeExplosion(self, explosion):
      "Returns an explosion to the free pool"
      self.__free.append(explosion)

explosions = ExplosionPool()

square = Square(3)
turtle = Turtle(CrossSectionTool(material, square))
turtle.stamp()
turtle.forward(3)
turtle.stamp()
#explosionModel = turtle.tool.model

# initialize the sounds
audio = wurld.getAudioDriver()
rocketSound = audio.createSampleFromFile('sounds/rocket1i.wav')
explosionSound = audio.createSampleFromFile('sounds/r_exp3.wav')

def splat():
   global explosions
   print "SPLAT!"
   explosion = explosions.getExplosion()
   explosion.reset(missile.origin)

   # hide the missile
   missile.velocity = wurld.createVector(0, 0, 0)

   # play the explosion
   player = audio.createPlayer(explosionSound)
   player.setMoveable(explosion.moveable)
   player.start()

def fire(keyCode):
   camera = wurld.getGameEngine().getCameraPtr()

   # calculate the "forward" vector
   relForward = wurld.createVector(0, 0, -1)
   forward = wurld.createVector(0, 0, 0)
   rot = camera.getRotation()
   rot.rotate(relForward, forward)

   missile.origin = camera.getOrigin() + forward * 10

   velocity = forward * 100
   missile.velocity = velocity
   missile.collisionCallback = splat

   # rotate the model to the inverse of the camera rotation
   modelRot = wurld.createYawRotMatrix(0)
   modelRot.makeInverse(rot)
   missile.rotation = modelRot

   # make a sound
   audio.createPlayer(rocketSound).start()

   print 'player = %s, missile = %s, vel = %s' % \
      (vecToStr(camera.getOrigin()), vecToStr(missile.origin), 
       vecToStr(missile.velocity))

bindKey(ord('a'), fire)
